import java.util.ArrayList;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.*;
import javax.swing.Timer;
import java.util.Scanner;
import java.io.File;
import java.io.FileNotFoundException;

/**
 * Cabezal de la maquina
 * Maneja el movimiento, la lectura y escritura de simbolos
 * sobre la cinta, y el cambio de estado.
 */

class Head implements ActionListener {

/** 
 * Constructor del cabezal
 */
  public Head() {
    pos = 9;
    tape = new Tape();
    halt = false;

    timer = new Timer(500, this);
    tapeView = new JLabel(tape.toString(pos));
    final_state = new JLabel("");
    stateView = new JTextArea(4,15);
    stateView.setEditable(false);
    program_loaded = false;
  }

/** 
 * Compila los estados y se los da al cabezal
 * @param program La lista de estados escrita en texto plano
 * @param source El origen del texto, define el tipo de copilacion
 */
  public void Compile(String program, String  source) {
    Scanner in = null;
    if (source=="file") {
    File file = new File(program);
        try{
            in = new Scanner(file);
        } catch(FileNotFoundException exception){
            System.out.println("The file " + file.getPath() + " was not found.");
        }
    } else if (source=="string") {
        in = new Scanner(program);
    }

    states = Compiler.States(in);
    initial_state = Compiler.InitialState(in);
    accepting_states = (ArrayList<String>) Compiler.AcceptingStates(in).clone();

    tape = new Tape();
    pos = 9;
    ChangeState(initial_state);
    tapeView.setText(tape.toString(pos));
    final_state.setText("");
    program_loaded = true;
  }

/**
 * Se encarga de procesar el estado de la maquina
 * Obtiene la instruccion correspondiente
 * Escribe en la cinta, mueve el cabezal, y cambia el estado
 * Detetiene la ejecucion de la maquina si no encuetra la instruccion correspondiente
 */
  public void ProcessState() {
  	/*Obtiene la instruccion correspondiente*/
    String c = tape.GetCellValue(pos);
  	Instruction i = actual_state.GetInstructions(c);
    if (i!=null) {
      Write(i.GetSymbol());
      Move(i.GetMovement());
      ChangeState(i.GetNextState());
    }
    else {
      halt = true;
    }
  }

/**
 * Escribe un simbolo en la cinta
 * @param symbol El simbolo a escribir
 */
  public void Write(String symbol) {
    tape.Write(symbol,pos);
  }

/**
 * Cambia la posicion del cabezal y agranda la cinta si es necesario
 * @param dir La direccion en que debe moverse la maquina
 */
  public void Move(String dir) {
    if (dir.equals(">")) {
      pos+=1;
      if (tape.TapeSize()-pos-1 < 9) {
        tape.GrowBack();
      }
    }
    else if (dir.equals("<")) {
      pos-=1;
      if (pos < 9) {
        tape.GrowFront();
        pos+=1;
      }
    }

  }

/**
 * Busca y cambia el estado de la maquina
 * @param next_state El estado al que debe cambiar
 */
  public void ChangeState(String next_state) {
    for (int i = 0; i < states.size(); i++) {
      if (states.get(i).GetName().equals(next_state)) {
        actual_state = states.get(i);
        stateView.setText(actual_state.toString());
        stateView.insert("Actual State: "+actual_state.GetName()+"\n",0);
      }
    }
  }

/**
 * Se encarga de finalizar la ejecucion de la maquina
 * @return Indica si la ejecucion fue detenida
 */
  public Boolean Halted() {
    if (halt) {
        if (accepting_states.contains(actual_state.GetName())) {
          final_state.setText("Acepted");
        } else {
          final_state.setText("Rejected");
        }
    }
    return halt;
  }

/**
 * Se encarga de indicar que se debe procesar el estado, junto con actualizar
 * el estado de la cinta en pantalla.
 */
  public void actionPerformed(ActionEvent event) {
    if (Halted()) {
      timer.stop();
    }
    else {
      ProcessState();
    }
    tapeView.setText(tape.toString(pos));
  }

/**
 * Comienta o pausa la ejecucion de la maquina
 */
  public void StartStop() {
    if (!timer.isRunning()) {
      timer.start();
    }
    else {
      timer.stop();
    }
  }

/**
 * Carga la cinta con los simbolos de ingreso
 * @param tapeEntry La entrada de la cinta
 */
  public void LoadTape(String tapeEntry) {
    tape.Load(tapeEntry);
    pos = 9;
    ChangeState(initial_state);
    tapeView.setText(tape.toString(pos));
    halt = false;
  }

/**
 * Cambia la velocidad con la que se procesan los estados
 * @param speed La velocidad deseada
 */
  public void SetSpeed(float speed) {
    int deltaTime = (int) Math.round(1000/speed);
    timer = new Timer(deltaTime,this);
  }

/**
 * Verifica si se cargo algun programa a la maquina
 * @return Si hay un programa cargado
 */
  public Boolean ProgramLoaded(){
    return program_loaded;
  }

  private int pos;
  private State actual_state;
  private String initial_state;
  private ArrayList<State> states;
  private ArrayList<String> accepting_states;
  private Tape tape;
  private Boolean halt;
  private Boolean program_loaded;
  private Timer timer;
  public JLabel tapeView;
  public JLabel final_state;
  public JTextArea stateView;
}